/******************************************************************************
 *
 * Licensed Materials - Property of IBM
 *
 * (C) COPYRIGHT IBM Corp. 2005
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *
 *****************************************************************************/

/********************************************************************
 * selfCareServlet.java
 *
 * This servlet contains methods that control the self-care requirements
 * for the Servlet Portfolio Example.
 *
 * The user must provide a set of attributes that are considered a necessity
 * for self care.
 *
 * The attributes and data are displayed to the user via the selfcare.jsp.
 * The table that is used is dynamically created from the set of attributes
 * defined in the itim_expi.properties file.
 *
 * These attributes must correspond to the Person attribute names (see
 * the ITIM Schema for more information, and refer to the itim_expi.properties
 * file in the SelfCare attribute section). .
 *
 * A list of attributes is stored in the "attibutes.selfcare" variable (coma
 * delimitted).Each attribute, in turn, must have a counterpart variable that
 * defines the verbose text displayed on the selfRegistration.jsp
 *
 * Date: 5/2003
 *
********************************************************************/
package examples.expi;

import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Date;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;

import com.ibm.itim.apps.jaas.spi.PlatformPrincipal;
import javax.security.auth.Subject;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.itim.apps.identity.PersonMO;
import com.ibm.itim.apps.search.*;
import com.ibm.itim.common.AttributeValue;
import com.ibm.itim.dataservices.model.DistinguishedName;
import com.ibm.itim.dataservices.model.domain.Person;
import com.ibm.itim.apps.ApplicationException;
import com.ibm.itim.apps.AuthorizationException;
import com.ibm.itim.apps.PlatformContext;
import com.ibm.itim.apps.Request;
import com.ibm.itim.apps.SchemaViolationException;

/**
 * @version 	1.0
 *
 * The selfcare servlet builds the dynamic content based on the properties file (as specfied in expiUtils.java)
 * @author Vitek Boruvka
 */
public class selfCareServlet extends HttpServlet {
	private static expiUtil utilObject = null;
	private static PrintWriter out = null;

	/**
	* Summary of functionality:
	* 	1) validate all required data (includes subject, platform context, userID and person
	* 	2) get the Person object (and all attributs)
	* 	3) get the set of attributes to manipulate from the properties file
	* 	4) build table of data items to manipulate
	* @see javax.servlet.http.HttpServlet#void (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	*/
	public void doGet(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {
		req.setCharacterEncoding("UTF-8");

		try {
			utilObject = new expiUtil();
		} catch (Exception e) {
			e.printStackTrace();
		}

		HttpSession session = (HttpSession) req.getSession(false);
		if (session == null) {
			System.out.println(
				"EXPI: selfCareServlet:doGet() - no session - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		Subject subject = (Subject) session.getAttribute(expiUtil.SUBJECT);

		if (subject == null) {
			System.out.println(
				"EXPI: selfCareServlet:doGet() - no subject - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}
		PlatformContext platform =
			(PlatformContext) session.getAttribute(expiUtil.PLATFORM_CONTEXT);

		if (platform == null) {
			System.out.println(
				"EXPI: selfCareServlet:doGet() - no Platform Context object - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		String userID = (String) session.getAttribute(expiUtil.LOGON_ID);
		if (userID == null) {
			System.out.println(
				"EXPI: selfCareServlet:doGet() - no userID found - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		PersonMO personMo = (PersonMO) session.getAttribute(expiUtil.PERSONMO);

		if (personMo == null) {
			System.out.println(
				"EXPI: selfCareServlet:doGet() - no PersonMO object found - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		try {
			utilObject.printAttributes(personMo.getData());
		} catch (RemoteException e) {
			e.printStackTrace();
			System.out.println(
				"EXPI: selfCareServlet:doGet() - getData - RemoteException");
		} catch (ApplicationException e) {
			e.printStackTrace();
			System.out.println(
				"EXPI: selfCareServlet:doGet() - getData - ApplicationException");
		}


		setRequestAttributes(req, personMo);

		// if an error was passed in forward that to the SELF care page also

		String msg = (String)session.getAttribute(expiUtil.ERR_LABEL);

		if (msg == null)
			msg = "";

		utilObject.forward(
			req,
			resp,
			msg,
			utilObject.getProperty(expiUtil.SELFCARE_PAGE));
	}

	/**
	 * Method setRequestAttributes.
	 * @param req
	 * @param selfcareAttrs
	 * @param person
	 */
	public void setRequestAttributes(HttpServletRequest req, PersonMO personMo) {
		Person person = null;

		// 1st get the list of attributes that we need to manipulate
		// By Default this is contain in the properties variable SELFCARE_ATTRS
		// Add the attributes onto the request

		String selfcareAttrs = utilObject.getProperty(expiUtil.SELFCARE_ATTRS);
		System.out.println("selfcare attrs: " + selfcareAttrs);

		if (selfcareAttrs != null) {
			try {
				person = personMo.getData();
			} catch (RemoteException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: selfCareServlet:doGet() - getData - RemoteException");
			} catch (ApplicationException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: selfCareServlet:doGet() - getData - ApplicationException");
			}
		}

		utilObject.printAttributes(person);

		StringTokenizer st = new StringTokenizer(selfcareAttrs, ",");
		while (st.hasMoreTokens()) {
			String sAttr = st.nextToken();

			AttributeValue sValue;

			sValue = person.getAttribute(sAttr);
			if (sValue != null && !sValue.isValueEmpty()) {
				// debug only - print the attributes we're playing with

				System.out.println(
					"   Attr: " + sAttr + " = " + sValue.getString());
				req.setAttribute(sAttr, sValue.getString());
			}
			else
				req.setAttribute(sAttr, "");
		}
	}

	/**
	* @see javax.servlet.http.HttpServlet#void (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	*/
	public void doPost(HttpServletRequest req, HttpServletResponse resp)
		throws ServletException, IOException {
		req.setCharacterEncoding("UTF-8");

		if (utilObject == null) {
			System.out.println(
				"EXPI: selfCareServlet:doPost() - expected a utilObject - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		HttpSession session = (HttpSession) req.getSession(false);
		if (session == null) {
			System.out.println(
				"EXPI: selfCareServlet:doPost() - no session - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		PersonMO personMo = (PersonMO) session.getAttribute(expiUtil.PERSONMO);

		if (personMo == null) {
			System.out.println(
				"EXPI: selfCareServlet:doPost() - no PersonMO object found - redirecting to logon page");
			resp.sendRedirect(expiUtil.LOGON_SERVLET);
		}

		String selfcareAttrs = utilObject.getProperty(expiUtil.SELFCARE_ATTRS);
		System.out.println("selfcare attrs: " + selfcareAttrs);

		if (selfcareAttrs != null) {
			Person person = null;

			try {
				person = personMo.getData();
			} catch (RemoteException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: selfCareServlet:doGet() - getData - RemoteException");
			} catch (ApplicationException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: selfCareServlet:doGet() - getData - ApplicationException");
			}

			// in order to provide additional flexibility - additional attributes may be
			// added to the JSP that are NOT part of the property file.  Thus we enumerate
			// the paratemers rather than decipher then based only on the property file.

			Enumeration enum = req.getParameterNames();

			while (enum.hasMoreElements()) {
				String element = (String) enum.nextElement();
				String sValue = req.getParameter(element);

				System.out.println("Updating: " + element + ": " + sValue);

				AttributeValue attrValue;

				attrValue = new AttributeValue(element, sValue);
				person.setAttribute(attrValue);
			}

			try {
				Request request = personMo.update(person, new Date());

				utilObject.forward(req,resp,"",utilObject.getProperty(expiUtil.SELFCARESUB_PAGE));
			} catch (AuthorizationException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: personMo.update() - getData - AuthorizationException");

				setRequestAttributes(req, personMo);

				utilObject.forward(req,resp,"Error - AuthorizationException",utilObject.getProperty(expiUtil.SELFCARE_PAGE));
			} catch (SchemaViolationException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: personMo.update() - getData - SchemaViolationException");
				utilObject.forward(req,resp,"Error - SchemaViolationException",utilObject.getProperty(expiUtil.SELFCARE_PAGE));
			} catch (RemoteException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: personMo.update) - getData - RemoteException");
				utilObject.forward(req,resp,"Error - RemoteException",utilObject.getProperty(expiUtil.SELFCARE_PAGE));
			} catch (ApplicationException e) {
				e.printStackTrace();
				System.out.println(
					"EXPI: personMo.update() - getData - ApplicationException");
				utilObject.forward(req,resp,"Error - ApplicationException",utilObject.getProperty(expiUtil.SELFCARE_PAGE));
			}
		}

	}
}
